==> chrobject-0.1.4.tgz: package/lib/appservices/EntryAppService.js <==
/**
 *  Creator: Christian Hotz
 *  Company: hydra newmedia GmbH
 *  Date: 11.06.16
 *
 *  Copyright hydra newmedia GmbH
 */
"use strict";
/**
 *  Imports
 */
var _ = require('lodash');
var Snapshot_1 = require('../utils/Snapshot');
var Diff_1 = require('../utils/Diff');
var DeepDiff_1 = require('../utils/DeepDiff');
var EntryAppService = (function () {
    function EntryAppService(entity, storage) {
        this.entity = entity;
        this.storage = storage;
    }
    EntryAppService.prototype.saveSnapshotAndDiff = function (obj, creator, timestamp, callback) {
        var _this = this;
        var id = _.get(obj, this.entity.idPath);
        this.storage.findLatestSnapshotBefore(id, timestamp, this.entity, function (err, oldSnap) {
            if (err) {
                return callback(err);
            }
            var newSnap = new Snapshot_1.Snapshot(obj, _this.entity, creator, timestamp);
            _this.storage.insertSnapshot(newSnap, function (err, newSnap) {
                var diff;
                if (err) {
                    return callback(err);
                }
                else if (oldSnap) {
                    diff = _this.diff(oldSnap, newSnap);
                }
                else {
                    var blankSnap = new Snapshot_1.Snapshot({}, _this.entity, creator, timestamp);
                    diff = _this.diff(blankSnap.setObjId(newSnap.objId), newSnap);
                }
                diff.linkToId(newSnap.id);
                _this.storage.insertDiff(diff, function (err, diff) {
                    if (callback) {
                        callback(err, { snapshot: newSnap, diff: diff });
                    }
                });
            });
        });
    };
    EntryAppService.prototype.saveSnapshot = function (obj, creator, timestamp, callback) {
        var newSnap = new Snapshot_1.Snapshot(obj, this.entity, creator, timestamp);
        this.storage.insertSnapshot(newSnap, callback);
    };
    EntryAppService.prototype.saveDiff = function (obj, creator, timestamp, callback) {
        var _this = this;
        var id = _.get(obj, this.entity.idPath);
        this.storage.findLatestSnapshotBefore(id, timestamp, this.entity, function (err, oldSnap) {
            if (err) {
                return callback(err);
            }
            var newSnap = new Snapshot_1.Snapshot(obj, _this.entity, creator, timestamp, oldSnap ? oldSnap.id : undefined);
            _this.storage.upsertSnapshot(newSnap, function (err, newSnap) {
                var diff;
                if (err) {
                    return callback(err);
                }
                else if (oldSnap) {
                    diff = _this.diff(oldSnap, newSnap);
                }
                else {
                    var blankSnap = new Snapshot_1.Snapshot({}, _this.entity, creator, timestamp);
                    diff = _this.diff(blankSnap.setObjId(newSnap.objId), newSnap);
                }
                _this.storage.insertDiff(diff, callback ? callback : undefined);
            });
        });
    };
    EntryAppService.prototype.diff = function (snapOne, snapTwo) {
        // Check if snapshots are of same object
        if (snapOne.objId !== snapTwo.objId) {
            throw new Error('Diffed snapshots are not of same object');
        }
        // Check if snapshots are of same entity
        if (!snapOne.entity.equals(snapTwo.entity)) {
            throw new Error('Diffed snapshots are not of same entity');
        }
        // swap snapshots if snapTwo was taken before snapOne
        if (snapOne.timestamp > snapTwo.timestamp) {
            var tmp = snapOne;
            snapOne = snapTwo;
            snapTwo = tmp;
        }
        // diff between the two snapshots
        var diffObj = this.deepDiff(snapOne.obj, snapTwo.obj);
        return new Diff_1.Diff(diffObj, snapTwo.objId, snapTwo.entity, snapTwo.creator, snapTwo.timestamp);
    };
    EntryAppService.prototype.deepDiff = function (one, two, path) {
        if (path === void 0) { path = ''; }
        var result = [];
        for (var _i = 0, _a = _.keys(one); _i < _a.length; _i++) {
            var key = _a[_i];
            var concatPath = path ? path + '.' + key : key;
            if (_.isPlainObject(one[key])) {
                if (!_.has(two, key)) {
                    result.push(new DeepDiff_1.DeepDiff('deleted', concatPath, one[key], null));
                }
                else {
                    result = _.concat(result, this.deepDiff(one[key], two[key], path ? path + '.' + key : key));
                }
            }
            else if (_.isBoolean(one[key]) || _.isDate(one[key]) || _.isNumber(one[key])
                || _.isNull(one[key]) || _.isRegExp(one[key]) || _.isString(one[key])) {
                if (!_.has(two, key)) {
                    result.push(new DeepDiff_1.DeepDiff('deleted', concatPath, one[key], null));
                }
                else if (_.get(one, key) !== _.get(two, key)) {
                    result.push(new DeepDiff_1.DeepDiff('edited', concatPath, one[key], two[key]));
                }
            }
            else if (_.isArray(one[key]) && _.isArray(two[key]) && !_.isEqual(one[key], two[key])) {
                result.push(new DeepDiff_1.DeepDiff('array', concatPath, one[key], two[key]));
            }
            else if (!_.has(two, key)) {
                result.push(new DeepDiff_1.DeepDiff('deleted', concatPath, one[key], null));
            }
        }
        for (var _b = 0, _c = _.keys(two); _b < _c.length; _b++) {
            var key = _c[_b];
            var concatPath = path ? path + '.' + key : key;
            if (!_.has(one, key)) {
                if (_.isPlainObject(two[key]) || _.isBoolean(two[key]) || _.isDate(two[key]) || _.isNumber(two[key])
                    || _.isNull(two[key]) || _.isRegExp(two[key]) || _.isString(two[key]) || _.isArray(two[key])) {
                    result.push(new DeepDiff_1.DeepDiff('created', concatPath, null, two[key]));
                }
            }
        }
        return result;
    };
    return EntryAppService;
}());
exports.EntryAppService = EntryAppService;

==> chrobject-0.2.0.tgz: package/lib/appservices/EntryAppService.js <==
/**
 *  Creator: Christian Hotz
 *  Company: hydra newmedia GmbH
 *  Date: 11.06.16
 *
 *  Copyright hydra newmedia GmbH
 */
"use strict";
/**
 *  Imports
 */
var _ = require('lodash');
var Snapshot_1 = require('../utils/Snapshot');
var Diff_1 = require('../utils/Diff');
var DeepDiff_1 = require('../utils/DeepDiff');
var EntryAppService = (function () {
    function EntryAppService(entity, storage) {
        this.entity = entity;
        this.storage = storage;
    }
    EntryAppService.prototype.saveSnapshotAndDiff = function (obj, creator, timestamp, callback) {
        var _this = this;
        var id = _.get(obj, this.entity.idPath);
        this.storage.findLatestSnapshotBefore(id, timestamp, this.entity, function (err, oldSnap) {
            if (err) {
                return callback(err);
            }
            var newSnap = new Snapshot_1.Snapshot(obj, _this.entity, creator, timestamp);
            var diff;
            if (oldSnap) {
                diff = _this.diff(oldSnap, newSnap);
            }
            else {
                var blankSnap = new Snapshot_1.Snapshot({}, _this.entity, creator, timestamp);
                diff = _this.diff(blankSnap.setObjId(newSnap.objId), newSnap);
            }
            if (!diff.isEmpty()) {
                _this.storage.insertSnapshot(newSnap, function (err, newSnap) {
                    if (err) {
                        return callback(err);
                    }
                    diff.linkToId(newSnap.id);
                    _this.storage.insertDiff(diff, function (err, diff) {
                        if (callback) {
                            callback(err, { snapshot: newSnap, diff: diff });
                        }
                    });
                });
            }
            else {
                callback(null, { snapshot: null, diff: null });
            }
        });
    };
    EntryAppService.prototype.saveSnapshot = function (obj, creator, timestamp, callback) {
        var _this = this;
        var id = _.get(obj, this.entity.idPath);
        this.storage.findLatestSnapshotBefore(id, timestamp, this.entity, function (err, oldSnap) {
            if (err) {
                return callback(err);
            }
            var newSnap = new Snapshot_1.Snapshot(obj, _this.entity, creator, timestamp);
            var diff;
            if (oldSnap) {
                diff = _this.diff(oldSnap, newSnap);
            }
            else {
                var blankSnap = new Snapshot_1.Snapshot({}, _this.entity, creator, timestamp);
                diff = _this.diff(blankSnap.setObjId(newSnap.objId), newSnap);
            }
            if (!diff.isEmpty()) {
                _this.storage.insertSnapshot(newSnap, callback);
            }
            else {
                callback(null, null);
            }
        });
    };
    EntryAppService.prototype.saveDiff = function (obj, creator, timestamp, callback) {
        var _this = this;
        var id = _.get(obj, this.entity.idPath);
        this.storage.findLatestSnapshotBefore(id, timestamp, this.entity, function (err, oldSnap) {
            if (err) {
                return callback(err);
            }
            var newSnap = new Snapshot_1.Snapshot(obj, _this.entity, creator, timestamp, oldSnap ? oldSnap.id : undefined);
            var diff;
            if (oldSnap) {
                diff = _this.diff(oldSnap, newSnap);
            }
            else {
                var blankSnap = new Snapshot_1.Snapshot({}, _this.entity, creator, timestamp);
                diff = _this.diff(blankSnap.setObjId(newSnap.objId), newSnap);
            }
            if (!diff.isEmpty()) {
                _this.storage.upsertSnapshot(newSnap, function (err, newSnap) {
                    if (err) {
                        return callback(err);
                    }
                    else {
                        _this.storage.insertDiff(diff, callback ? callback : undefined);
                    }
                });
            }
            else {
                callback(null, null);
            }
        });
    };
    EntryAppService.prototype.diff = function (snapOne, snapTwo) {
        // Check if snapshots are of same object
        if (snapOne.objId !== snapTwo.objId) {
            throw new Error('Diffed snapshots are not of same object');
        }
        // Check if snapshots are of same entity
        if (!snapOne.entity.equals(snapTwo.entity)) {
            throw new Error('Diffed snapshots are not of same entity');
        }
        // swap snapshots if snapTwo was taken before snapOne
        if (snapOne.timestamp > snapTwo.timestamp) {
            var tmp = snapOne;
            snapOne = snapTwo;
            snapTwo = tmp;
        }
        // diff between the two snapshots
        var diffObj = this.deepDiff(snapOne.obj, snapTwo.obj);
        return new Diff_1.Diff(diffObj, snapTwo.objId, snapTwo.entity, snapTwo.creator, snapTwo.timestamp);
    };
    EntryAppService.prototype.deepDiff = function (one, two, path) {
        if (path === void 0) { path = ''; }
        var result = [];
        for (var _i = 0, _a = _.keys(one); _i < _a.length; _i++) {
            var key = _a[_i];
            var concatPath = path ? path + '.' + key : key;
            if (_.isPlainObject(one[key])) {
                if (!_.has(two, key)) {
                    result.push(new DeepDiff_1.DeepDiff('deleted', concatPath, one[key], null));
                }
                else {
                    result = _.concat(result, this.deepDiff(one[key], two[key], path ? path + '.' + key : key));
                }
            }
            else if (_.isBoolean(one[key]) || _.isDate(one[key]) || _.isNumber(one[key])
                || _.isNull(one[key]) || _.isRegExp(one[key]) || _.isString(one[key])) {
                if (!_.has(two, key)) {
                    result.push(new DeepDiff_1.DeepDiff('deleted', concatPath, one[key], null));
                }
                else if (_.get(one, key) !== _.get(two, key)) {
                    result.push(new DeepDiff_1.DeepDiff('edited', concatPath, one[key], two[key]));
                }
            }
            else if (_.isArray(one[key]) && _.isArray(two[key]) && !_.isEqual(one[key], two[key])) {
                result.push(new DeepDiff_1.DeepDiff('array', concatPath, one[key], two[key]));
            }
            else if (!_.has(two, key)) {
                result.push(new DeepDiff_1.DeepDiff('deleted', concatPath, one[key], null));
            }
        }
        for (var _b = 0, _c = _.keys(two); _b < _c.length; _b++) {
            var key = _c[_b];
            var concatPath = path ? path + '.' + key : key;
            if (!_.has(one, key)) {
                if (_.isPlainObject(two[key]) || _.isBoolean(two[key]) || _.isDate(two[key]) || _.isNumber(two[key])
                    || _.isNull(two[key]) || _.isRegExp(two[key]) || _.isString(two[key]) || _.isArray(two[key])) {
                    result.push(new DeepDiff_1.DeepDiff('created', concatPath, null, two[key]));
                }
            }
        }
        return result;
    };
    return EntryAppService;
}());
exports.EntryAppService = EntryAppService;
